
import processing.opengl.*;
import javax.media.opengl.*; 
import javax.media.opengl.glu.*;
import com.sun.opengl.util.texture.*; 
import javax.vecmath.*;
import com.sun.opengl.util.BufferUtil;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.nio.ByteBuffer;
import java.nio.IntBuffer;
import java.nio.charset.Charset;
import java.util.ArrayList;

float FOVY = 45f;
float NEAR = .1f;
float FAR = 1000f;

float BLUR_OPACITY = .5f;
float BLUR_MARGIN = .5f;

PGraphicsOpenGL pgl;
GL gl;
GLU glu;
Camera cam;

int[] viewportBounds = new int[4];
TextureObj tex1 = null;
TextureObj tex2 = null;
TextureObj blurTex = null;
public int cols = 4;
public int rows = 4;
public Tile[][] tiles;
float sx = -2f;
float sy = -2f;
float xw = 4f;
float yh = 4f;
float xinc = xw / cols;
float yinc = yh / rows;
      
void setup() 
{
  size(860, 500, OPENGL); 
  
  tiles = new Tile[cols][rows];
  for (int i = 0; i < cols; i++)
  {
    for (int j = 0; j < rows; j++)
    {
      tiles[i][j] = new Tile();
      tiles[i][j].xpos = sx + (xinc * i);     
      tiles[i][j].ypos = sy + (yinc * j);     
      tiles[i][j].opacity = .5f;
    }
  }
  
  frame.setResizable(true); 
  viewportBounds = new int[]
  { 
    0,0, width, height               
  };

  tex1 = new TextureObj(this, "pehr.jpg");
  tex2 = new TextureObj(this, "andres.jpg");
  blurTex = new TextureObj(this, "gauss.jpg");

  cam = new Camera();

  background(0);
}

void initProjection(GL gl, GLU glu)
{
  gl.glViewport(0, 0, width, height);  
  gl.glMatrixMode(GL.GL_PROJECTION);
  gl.glLoadIdentity();
  glu.gluPerspective(FOVY, (float)width/(float)height, NEAR, FAR);
}

void initModelview(GL gl)
{
  gl.glMatrixMode(GL.GL_MODELVIEW);
  gl.glLoadIdentity();
}

int programID = -1;
int opacID = -1;
int blurOpacID = -1;
int blurTexID = -1;
int tex1ID = -1;
int tex2ID = -1;

boolean firstTime = true;

void draw()
{
  background(0); //clear screen

  //get OpenGL context
  pgl = (PGraphicsOpenGL) g; 
  gl = pgl.gl;
  glu = pgl.glu;

  //override Processing defaults to work with normal OpenGL matrices
  initProjection(gl, glu);
  initModelview(gl);

  //handle camera view
  cam.renderCam(gl, glu);

  if (firstTime == true)
  {
    //load in shader program
    programID = initShaders("multiText.vert", "multiText.frag");
   
    opacID = gl.glGetAttribLocation(programID, "opac");
    blurOpacID = gl.glGetAttribLocation(programID, "blurOpac");
    blurTexID = gl.glGetUniformLocation(programID, "blurTex"); 
    tex1ID = gl.glGetUniformLocation(programID, "tex1");
    tex2ID = gl.glGetUniformLocation(programID, "tex2");
    
    firstTime = false;
  }

  if (programID <= 0)
  {
    println("ERROR - couldn't load shader program!");
  }
  else
  { 
    //Bind GLSL Program to overwrite fixed functionality
    gl.glUseProgram(programID);
    
    gl.glActiveTexture(gl.GL_TEXTURE0);
    blurTex.texture.bind();
    gl.glEnable(gl.GL_TEXTURE_2D);
    gl.glActiveTexture(gl.GL_TEXTURE1);
    tex1.texture.bind();
    gl.glEnable(gl.GL_TEXTURE_2D);
    gl.glActiveTexture(gl.GL_TEXTURE2);
    tex2.texture.bind();

    gl.glEnable(gl.GL_TEXTURE_2D);

    gl.glUniform1i(blurTexID, 0);
    gl.glUniform1i(tex1ID, 1);
    gl.glUniform1i(tex2ID, 2);
   
    TextureCoords tc = tex1.texture.getImageTexCoords();


    float twInc = (tc.right() - tc.left()) / cols;
    float thInc = (tc.top() - tc.bottom()) / rows;
    float wInc = xw / cols;
    float hInc = yh / rows;

    gl.glEnable(gl.GL_BLEND);
    gl.glBegin(gl.GL_QUADS);

    for (int i = 0; i < cols; i++)
    {
      for (int j = 0; j < rows; j++)
      {
        //gl.glUniform1f(program.uniform("opacity"), grains[i][j].opacity);
        gl.glVertexAttrib1f(opacID, tiles[i][j].opacity);
        gl.glVertexAttrib1f(blurOpacID, tiles[i][j].blurOpacity);

        float margin = .0; //grains[i][j].margin;
        tiles[i][j].blurOpacity = BLUR_OPACITY;
        tiles[i][j].blurMarginX = BLUR_MARGIN;
        tiles[i][j].blurMarginY = BLUR_MARGIN;
        tiles[i][j].opacity += tiles[i][j].rate;
        if (tiles[i][j].opacity > 1f || tiles[i][j].opacity < 0f)
        {
          tiles[i][j].rate *= -1f;
        }
          drawMultiTexturedTile(
          gl,
          tiles[i][j].xpos, 
          tiles[i][j].ypos,
          wInc, 
          hInc,
         
          twInc * i, 
          twInc * (i + 1), 
          tc.bottom() + (thInc * j), 
          tc.bottom() + (thInc * (j + 1)),
          tiles[i][j].blurMarginX, 
          tiles[i][j].blurMarginY,
          tiles[i][j].opacity
            );

      }
    }

    gl.glEnd();

    gl.glActiveTexture(gl.GL_TEXTURE0);
    gl.glDisable(gl.GL_TEXTURE_2D);
    gl.glActiveTexture(gl.GL_TEXTURE1);
    gl.glDisable(gl.GL_TEXTURE_2D);
    gl.glActiveTexture(gl.GL_TEXTURE2);
    gl.glDisable(gl.GL_TEXTURE_2D);


    //Un-bind GLSL Program to return to fixed functionality
    gl.glUseProgram(0);
  }
}

void resetProcessingMatrices(GL gl, GLU glu)
{
  gl.glMatrixMode(GL.GL_PROJECTION);
  gl.glLoadIdentity();
  glu.gluPerspective(60, width/(float)height, .1f, 1000f);
  gl.glMatrixMode(GL.GL_MODELVIEW);
  gl.glLoadIdentity();
  glu.gluLookAt(width/2.0f, height/2.0f, (height/2.0f) / tan(PI*60.0f / 360.0f), width/2.0f, height/2.0f, 0, 0, 1, 0); 
}



void keyPressed()
{
  switch(key)
  {
  case 'a':
    cam.moveCamX(-cam.translateSpeed);
    break;
  case 'z':
    cam.moveCamX(cam.translateSpeed);
    break;
  case 's':
    cam.moveCamY(cam.translateSpeed);
    break;
  case 'x':
    cam.moveCamY(-cam.translateSpeed);
    break;
  case 'd':
    cam.moveCamZ(-cam.translateSpeed);
    break;
  case 'c':
    cam.moveCamZ(cam.translateSpeed);
    break;

  case 'g':
    cam.rotateCamY(cam.rotateSpeed);
    break;
  case 'b':
    cam.rotateCamY(-cam.rotateSpeed);
    break;
  case 'h':
    cam.rotateCamZ(-cam.rotateSpeed);
    break;
  case 'n':
    cam.rotateCamZ(cam.rotateSpeed);
    break;
  case 'j':
    cam.rotateCamX(-cam.rotateSpeed);
    break;
  case 'm':
    cam.rotateCamX(cam.rotateSpeed);
    break;

  case 'q':
    cam.resetCamera();
    break;

  case '1':  
    BLUR_OPACITY +=.01f;
    break;
  case '2':  
    BLUR_OPACITY -=.01f;  
    break;
    
  case '3':  
    BLUR_MARGIN +=.01f;
    break;
  case '4':  
    BLUR_MARGIN -=.01f;  
    break;
   }
}

void mouseDragged() 
{
  float rate = .5f;
  cam.rotateCamY((mouseX-pmouseX) * rate);
}




















